; *******************************************************************
; *** This software is copyright 2005 by Michael H Riley          ***
; *** You have permission to use, modify, copy, and distribute    ***
; *** this software so long as this copyright notice is retained. ***
; *** This software may not be used in commercial applications    ***
; *** without express written permission from the author.         ***
; *******************************************************************

include    bios.inc
include    kernel.inc

           org     8000h
           db      'XR',0
           dw      9000h
           dw      endrom+7000h
           dw      2000h
           dw      endrom-2000h
           dw      2000h
           db      0

           org     2000h
           br      start

include    date.inc

fildes:    db      0,0,0,0
           dw      dta
           db      0,0
           db      0
           db      0,0,0,0
           dw      0,0
           db      0,0,0,0
block:     db      0

dta:       equ     7000h
dtapage:   equ     070h
stack:     equ     7fffh

start:     ghi     ra                  ; copy argument address to rf
           phi     rf
           glo     ra
           plo     rf
loop1:     lda     ra                  ; look for first less <= space
           smi     33
           bdf     loop1
           dec     ra                  ; backup to char
           ldi     0                   ; need proper termination
           str     ra
           ldi     high fildes         ; get file descriptor
           phi     rd
           ldi     low fildes
           plo     rd
           ldi     0                   ; no special flags
           plo     r7
           sep     scall               ; attempt to open file
           dw      o_open
           bnf     opened              ; jump if file opened
           ldi     high errmsg         ; point to error message
           phi     rf
           ldi     low errmsg
           plo     rf
           sep     scall               ; display error message
           dw      o_msg
           lbr     o_wrmboot           ; return to Elf/OS
errmsg:    db      'file error',10,13,0
opened:    ldi     1                   ; setup block number
           plo     r9

filelp:    ldi     high rxbuffer       ; point to buffer
           phi     rf
           ldi     low rxbuffer
           plo     rf
           ldi     0                   ; need to read 128 bytes
           phi     rc
           ldi     128
           plo     rc
clearlp:   ldi     0                   ; clear out buffer
           str     rf
           inc     rf
           dec     rc
           glo     rc
           lbnz    clearlp
           ldi     high rxbuffer       ; point to buffer
           phi     rf
           ldi     low rxbuffer
           plo     rf
           ldi     0                   ; need to read 128 bytes
           phi     rc
           ldi     128
           plo     rc
           sep     scall               ; write buffer to file
           dw      o_read
           glo     rc                  ; see if bytes were read
           lbz     filedn              ; jump if not
           ldi     high rxbuffer       ; point to buffer
           phi     rf
           ldi     low rxbuffer
           plo     rf
           ldi     0                   ; need to send 128 bytes
           phi     rc
           ldi     128
           plo     rc
           sep     scall               ; send the block
           dw      sendblk
           lbr     filelp              ; loop back until full file sent

filedn:    sep     scall               ; close file
           dw      o_close
endlp:     ldi     4h                  ; signal EOT
           sep     scall
           dw      o_type
           sep     scall               ; read byte from receiver
           dw      o_read
           smi     6h                  ; must be an ACK
           lbnz    endlp
           lbr     o_wrmboot           ; and return to os

; **********************************
; *** Send an xmodem block       ***
; *** RF - pointer to data block ***
; *** RC - bytes to send         ***
; **********************************
sendblk:   ghi     rf                  ; make copy of block address
           phi     r7
           glo     rf
           plo     r7
           ghi     rc                  ; copy block size
           phi     rb
           glo     rc
           plo     rb
           glo     rc                  ; see if 128 byte block
           smi     128
           lbz     blk128              ; jump if so
           ldi     2                   ; send STX (1024 byte blocks)
           lskp
blk128:    ldi     1                   ; send SOH (128 byte blocks)
           sep     scall
           dw      o_type
           glo     r9                  ; get block number
           sep     scall               ; and transmit it
           dw      o_type
           glo     r9                  ; send inverse
           xri     255                 ; invert it
           sep     scall
           dw      o_type
           ldi     0                   ; clear checksum
           plo     r8
sendlp:    sex     r7                  ; need to add byte into checksum
           glo     r8
           add 
           plo     r8
           sex     r2                  ; x back to stack
           lda     r7                  ; get byte to send
           sep     scall               ; and send it
           dw      o_type
           dec     rb                  ; decrement block size count
           glo     rb                  ; see if done
           lbnz    sendlp              ; jump if not
           ghi     rb                  ; check high byte as well
           lbnz    sendlp
           glo     r8                  ; get checksum byte
           sep     scall               ; and send it
           dw      o_type
           sep     scall               ; get result from receiver
           dw      o_read
           smi     6h                  ; check for ACK
           lbnz    sendblk             ; resend block if not
           inc     r9                  ; increment block count
           sep     sret                ; and return to caller



endrom:    equ     $

rxbuffer:  equ     $

